@@ -1262,16 +1262,17 @@ use_pictograms (HTMLEngine *e)
 		html_cursor_forward_n (e->cursor, e, len);
 
 		filename = g_strconcat ("file://" ICONDIR "/", picto_images [-state - 1], NULL);
-		picto = html_image_new (e->image_factory, filename, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
+		picto = html_image_new (html_engine_get_image_factory (e), filename, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
 					HTML_VALIGN_MIDDLE, FALSE);
 		html_image_set_alt (HTML_IMAGE (picto), alt);
+		html_object_set_data (HTML_OBJECT (picto), "picto", alt);
 		g_free (alt);
 		html_engine_paste_object (e, picto, html_object_get_length (picto));
 	}
 }
 
 void
-html_engine_insert_text_with_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
+html_engine_insert_text_with_extra_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
 {
 	gchar *nl;
 	gint alen;
@@ -1299,10 +1300,8 @@ html_engine_insert_text_with_attributes (HTMLEngine *e, const gchar *text, guint
 				html_engine_set_insertion_link (e, NULL, NULL);
 
 			o = html_engine_new_text (e, text, alen);
-			if (attrs) {
-				pango_attr_list_splice (HTML_TEXT (o)->attr_list, attrs, 0, HTML_TEXT (o)->text_len);
-				gtk_html_debug_list_text_attrs (HTML_TEXT (o));
-			}
+			if (attrs)
+				HTML_TEXT (o)->extra_attr_list = pango_attr_list_copy (attrs);
 			html_text_convert_nbsp (HTML_TEXT (o), TRUE);
 
 			if (alen == 1 && html_is_in_word (html_text_get_char (HTML_TEXT (o), 0))
@@ -1329,11 +1328,11 @@ html_engine_insert_text_with_attributes (HTMLEngine *e, const gchar *text, guint
 void
 html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 {
-	html_engine_insert_text_with_attributes (e, text, len, NULL);
+	html_engine_insert_text_with_extra_attributes (e, text, len, NULL);
 }
 
 void
-html_engine_paste_text_with_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
+html_engine_paste_text_with_extra_attributes (HTMLEngine *e, const gchar *text, guint len, PangoAttrList *attrs)
 {
 	gchar *undo_name = g_strdup_printf ("Paste text: '%s'", text);
 	gchar *redo_name = g_strdup_printf ("Unpaste text: '%s'", text);
@@ -1342,14 +1341,35 @@ html_engine_paste_text_with_attributes (HTMLEngine *e, const gchar *text, guint
 	g_free (undo_name);
 	g_free (redo_name);
 	html_engine_delete (e);
-	html_engine_insert_text_with_attributes (e, text, len, attrs);
+	html_engine_insert_text_with_extra_attributes (e, text, len, attrs);
 	html_undo_level_end (e->undo);
 }
 
 void
 html_engine_paste_text (HTMLEngine *e, const gchar *text, guint len)
 {
-	html_engine_paste_text_with_attributes (e, text, len, NULL);
+	html_engine_paste_text_with_extra_attributes (e, text, len, NULL);
+}
+
+void
+html_engine_paste_link (HTMLEngine *e, const char *text, int len, const char *complete_url)
+{
+	char *url, *target;
+
+	if (len == -1)
+		len = g_utf8_strlen (text, -1);
+
+	url = g_strdup (complete_url);
+	target = strrchr (url, '#');
+	if (target) {
+		*target = 0;
+		target ++;
+	}
+
+	html_engine_paste_text (e, text, len);
+	html_text_add_link (HTML_TEXT (e->cursor->object), e, url, target, e->cursor->offset - len, e->cursor->offset);
+
+	g_free (url);
 }
 
 void
@@ -1380,6 +1400,24 @@ html_engine_delete_n (HTMLEngine *e, guint len, gboolean forward)
 		html_engine_set_mark (e);
 		html_engine_update_selection_if_necessary (e);
 		html_engine_freeze (e);
+		/* Remove magic smiley */
+		if (!forward && len == 1 && gtk_html_get_magic_smileys (e->widget)) {
+			HTMLObject *object = html_object_get_tail_leaf (e->cursor->object);
+
+			if (HTML_IS_IMAGE (object) && html_object_get_data (object, "picto") != NULL) {
+				gchar *picto = g_strdup (html_object_get_data (object, "picto"));
+				html_undo_level_begin (e->undo, "Remove Magic Smiley", "Undo Remove Magic Smiley");
+				html_cursor_backward (e->cursor, e);
+				html_engine_delete (e);
+				html_engine_insert_text (e, picto, -1);
+				html_undo_level_end (e->undo);
+				g_free (picto);
+
+				html_engine_unblock_selection (e);
+				html_engine_thaw (e);
+				return;
+			}
+		}
 		while (len != 0) {
 			if (forward)
 				html_cursor_forward (e->cursor, e);
