@@ -211,6 +211,7 @@ void              html_text_add_link_full                (HTMLText           *te
 							  gint                end_index,
 							  gint                start_offset,
 							  gint                end_offset);
+void              html_text_remove_links                 (HTMLText           *text);
 gboolean          html_text_get_link_rectangle           (HTMLText           *text,
 							  HTMLPainter        *painter,
 							  gint                offset,
@@ -309,7 +310,11 @@ void               html_text_calc_text_size        (HTMLText              *t,
 						    gint                  *width,
 						    gint                  *asc,
 						    gint                  *dsc);
-gboolean           html_text_is_line_break         (PangoLogAttr           attr);
+
+gboolean  html_text_is_line_break                (PangoLogAttr  attr);
+void      html_text_remove_unwanted_line_breaks  (char         *s,
+						  int           len,
+						  PangoLogAttr *attrs);
 
 typedef HTMLObject * (* HTMLTextHelperFunc)       (HTMLText *, gint begin, gint end);
 HTMLObject *html_text_op_copy_helper    (HTMLText           *text,
